#include "crypto.h"
#include "config.h"
#include <string.h>
#include <mbedtls/md.h>
#include <esp_random.h>
#include <bootloader_random.h>

void enableCrypto() {
    bootloader_random_enable();
}

uint32_t randomChallenge() {
    return esp_random();
}

bool authenticateData(const uint8_t* data, size_t dataLength, uint8_t* mac, const uint8_t* key) {
    mbedtls_md_context_t ctx;
    mbedtls_md_type_t md_type = MBEDTLS_MD_SHA256;
    int result;

    mbedtls_md_init(&ctx);
    result = mbedtls_md_setup(&ctx, mbedtls_md_info_from_type(md_type), 1);
    if (result) {
        return false;
    }
    result = mbedtls_md_hmac_starts(&ctx, key, keySize);
    if (result) {
        return false;
    }
    result = mbedtls_md_hmac_update(&ctx, data, dataLength);
    if (result) {
        return false;
    }
    result = mbedtls_md_hmac_finish(&ctx, mac);
    if (result) {
        return false;
    }
    mbedtls_md_free(&ctx);
    return true;
}

bool authenticateMessage(Message* message, uint8_t* mac, const uint8_t* key) {
    return authenticateData((const uint8_t*) message, MESSAGE_CONTENT_SIZE, mac, key);
}

bool authenticateMessage(SignedMessage* message, const uint8_t* key) {
    return authenticateMessage(&message->message, message->mac, key);
}

bool isAuthenticMessage(SignedMessage* message, const uint8_t* key) {
    uint8_t mac[SHA256_MAC_SIZE];
    if (!authenticateMessage(&message->message, mac, key)) {
        return false;
    }
    return memcmp(mac, message->mac, SHA256_MAC_SIZE) == 0;
}